from sqlalchemy import *
from migrate import *
import migrate.changeset

class Dictionaries(object):
    def __init__(self, meta):
        self.accumulatesprices = Table('accumulatesprices', meta,
            Column('code', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=True),
        )
        self.actionparameter = Table('actionparameter', meta,
            Column('actionparametercode', Integer, nullable=True, primary_key=True, autoincrement=True),
            Column('parameterorder', Integer, nullable=True),
            Column('parametervalue', Text, nullable=True),
            Column('cmactioncode', Integer, nullable=True),
        )
        self.aspect = Table('aspect', meta,
            Column('aspectcode', Integer, nullable=False, primary_key=True),
            Column('aspectname', String(255), nullable=True),
            Column('aspectorder', Integer, nullable=True),
            Column('aspectschemecode', Integer, nullable=True),
        )
        self.aspectscheme = Table('aspectscheme', meta,
            Column('aspectschemecode', Integer, nullable=False, primary_key=True),
            Column('aspectschemename', String(255), nullable=True),
        )
        self.aspectvalue = Table('aspectvalue', meta,
            Column('aspectvaluecode', Integer, nullable=False, primary_key=True),
            Column('aspectvaluename', String(255), nullable=True),
            Column('aspectcode', Integer, nullable=False),
        )
        self.aspectvalueset = Table('aspectvalueset', meta,
            Column('aspectvaluesetcode', Integer, nullable=False, primary_key=True),
            Column('cquant', Numeric(precision=19, scale=2), nullable=True),
            Column('price', Numeric(precision=19, scale=2), nullable=True),
            Column('tmc_code', String(255), nullable=True),
            Column('aspectschemecode', Integer, nullable=False),
            Column('measure', Integer, nullable=True),
        )
        self.aspectvaluesetvalues = Table('aspectvaluesetvalues', meta,
            Column('aspectcode', Integer, nullable=False),
            Column('aspectvaluecode', Integer, nullable=False),
            Column('aspectvaluesetcode', Integer, nullable=False),
        )
        self.campaign = Table('campaign', meta,
            Column('campaignid', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('name', String(100), nullable=True),
        )
        self.card = Table('card', meta,
            Column('idcard', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('idcardgroup', Integer, nullable=True),
            Column('idclient', Integer, nullable=True),
            Column('number', String(200), nullable=True),
            Column('validitydatebeg', Date, nullable=True),
            Column('validitydateend', Date, nullable=True),
            Column('cardSum', Numeric(precision=20, scale=2), nullable=True),
        )
        self.cardgroup = Table('cardgroup', meta,
            Column('idcardgroup', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('name', String(200), nullable=True),
            Column('text', String(200), nullable=True),
            Column('cardmode', Integer, nullable=True),
            Column('prefixbeg', String(200), nullable=True),
            Column('prefixend', String(200), nullable=True),
            Column('lenbeg', Integer, nullable=True),
            Column('lenend', Integer, nullable=True),
            Column('regexpattern', String(200), nullable=True),
            Column('inputmask', Integer, nullable=True),
            Column('regexrule', String(200), nullable=True),
            Column('valutcode', Integer, nullable=True),
            Column('discounttype', Integer, nullable=True),
            Column('discountvalue', Integer, nullable=True),
        )
        self.client = Table('client', meta,
            Column('idclient', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('name', String(200), nullable=True),
            Column('text', String(200), nullable=True),
            Column('sex', Integer, nullable=True),
            Column('birthday', Date, nullable=True),
            Column('specialdate1', Date, nullable=True),
            Column('specialdate2', Date, nullable=True),
            Column('specialdate3', Date, nullable=True),
            Column('specialdate1name', String(200), nullable=True),
            Column('specialdate2name', String(200), nullable=True),
            Column('specialdate3name', String(200), nullable=True),
            Column('zipcode', String(200), nullable=True),
            Column('address', String(200), nullable=True),
            Column('email', String(200), nullable=True),
            Column('webpage', String(200), nullable=True),
            Column('phonenumber', String(200), nullable=True),
            Column('inn', String(200), nullable=True),
            Column('document', String(200), nullable=True),
            Column('okpo', String(200), nullable=True),
            Column('okpd', String(200), nullable=True),
            Column('occupation', String(200), nullable=True),
            Column('childrencount', Integer, nullable=True),
            Column('extendedoptions', Text, nullable=True),
        )
        self.cmaction = Table('cmaction', meta,
            Column('cmactioncode', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('cmactionname', String(255), nullable=True),
            Column('commandcode', Integer, nullable=True),
        )
        self.cmkeys = Table('cmkeys', meta,
            Column('keycode', Integer, nullable=False, primary_key=True),
            Column('command', String(60), nullable=True),
            Column('params', String(512), nullable=True),
        )
        self.command = Table('command', meta,
            Column('commandcode', Integer, nullable=False, primary_key=True),
            Column('commandname', String(200), nullable=True),
            Column('description', String(200), nullable=True),
        )
        self.dept = Table('dept', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('name', String(32), nullable=True),
            Column('bcode', String(20), nullable=True),
            Column('vatcode1', Integer, nullable=True),
            Column('vatcode2', Integer, nullable=True),
            Column('vatcode3', Integer, nullable=True),
            Column('vatcode4', Integer, nullable=True),
            Column('vatcode5', Integer, nullable=True),
            Column('mode1', Integer, nullable=True),
            Column('mode2', Integer, nullable=True),
            Column('pricetype', Integer, nullable=True),
            Column('sum_limit', Numeric(precision=13, scale=2), nullable=True),
        )
        self.disccard = Table('disccard', meta,
            Column('cardnumber', String(128), nullable=False, primary_key=True),
            Column('name', String(100), nullable=True),
            Column('rtext', String(30), nullable=True),
            Column('cardsum', Numeric(precision=17, scale=2), nullable=True),
            Column('discasheme', Integer, nullable=True),
            Column('denycard', Integer, nullable=True),
            Column('notaccumulate', Integer, nullable=True),
            Column('positionmode', Integer, nullable=True),
            Column('ispaycard', Integer, nullable=True),
            Column('allpositions', Integer, nullable=True),
            Column('msgsheme', Integer, nullable=True),
            Column('rcount', Numeric(precision=20,scale=0), nullable=True),
            Column('acdiscscheme', Integer, nullable=True),
            Column('cardmode', Integer, nullable=True),
            Column('bonussum', Numeric(precision=17, scale=2), nullable=True),
            Column('bonuscode', Integer, nullable=True),
            Column('groupcode', Integer, nullable=True),
            Column('birthday', Date, nullable=True),
            Column('acdiscasheme', Integer, nullable=True),
        )
        self.disccardaccumulate = Table('disccardaccumulate', meta,
            Column('disccode', Integer, nullable=True),
            Column('name', String(30), nullable=True),
            Column('rtext', String(30), nullable=True),
            Column('discperc', Numeric(precision=5, scale=1), nullable=True),
            Column('isbegsum', Integer, nullable=True),
            Column('begsum', Numeric(precision=17, scale=2), nullable=True),
            Column('isendsum', Integer, nullable=True),
            Column('endsum', Numeric(precision=17, scale=2), nullable=True),
            Column('isbegrcount', Integer, nullable=True),
            Column('begrcount', Numeric(precision=20,scale=0), nullable=True),
            Column('isendrcount', Integer, nullable=True),
            Column('endrcount', Numeric(precision=20,scale=0), nullable=True),
            Column('discscheme', Integer, nullable=True),
            Column('discschemename', String(30), nullable=True),
        )
        self.discountsauto = Table('discountsauto', meta,
            Column('shemecode', Integer, nullable=False, primary_key=True),
            Column('disccode', Integer, nullable=False, primary_key=True),
            Column('name', String(30), nullable=True),
            Column('disctype', Integer, nullable=True),
            Column('discmode', Integer, nullable=True),
            Column('discsize', Numeric(precision=15, scale=2), nullable=True),
            Column('rtext', String(30), nullable=True),
            Column('date_beg', Date, nullable=True),
            Column('date_end', Date, nullable=True),
            Column('time_beg', Time(timezone=False), nullable=True),
            Column('time_end', Time(timezone=False), nullable=True),
            Column('days_beg', Integer, nullable=True),
            Column('days_end', Integer, nullable=True),
            Column('quant_beg', Numeric(precision=17, scale=3), nullable=True),
            Column('quant_end', Numeric(precision=17, scale=3), nullable=True),
            Column('sum_beg', Numeric(precision=17, scale=2), nullable=True),
            Column('sum_end', Numeric(precision=17, scale=2), nullable=True),
            Column('pricecode', Integer, nullable=True),
            Column('rsummode', Integer, nullable=True),
            Column('cardgroupcode', Integer, nullable=True),
            Column('birthdaymode', Integer, nullable=True),
        )
        self.discountsfixed = Table('discountsfixed', meta,
            Column('disccode', Integer, nullable=False, primary_key=True),
            Column('name', String(25), nullable=True),
            Column('disctype', Integer, nullable=True),
            Column('discmode', Integer, nullable=True),
            Column('discsize', Numeric(precision=15, scale=2), nullable=True),
            Column('rtext', String(30), nullable=True),
            Column('cardpref', String(24), nullable=True),
            Column('receiptmode', Integer, nullable=True),
            Column('positionmode', Integer, nullable=True),
            Column('cardonly', Integer, nullable=True),
            Column('cardsum_beg', Numeric(precision=18, scale=2), nullable=True),
            Column('cardsum_end', Numeric(precision=18, scale=2), nullable=True),
        )
        self.discountsystem = Table('discountsystem', meta,
            Column('discsystem', Text, nullable=True),
            Column('name', String(100), nullable=True),
            Column('discountsystemid', Integer, nullable=False, primary_key=True, autoincrement=True),
        )
        self.goodstimerestrict = Table('goodstimerestrict', meta,
            Column('bcode', String(100), nullable=True),
            Column('code', String(100), nullable=True),
            Column('flag', Integer, nullable=True),
            Column('text', String(100), nullable=True),
            Column('time_beg', DateTime, nullable=True),
            Column('time_end', DateTime, nullable=True),
        )
        self.groups = Table('groups', meta,
            Column('class', Integer, nullable=True),
            Column('sclass', Integer, nullable=True),
            Column('name', String(20), nullable=True),
            Column('pricetype', Integer, nullable=True),
        )
        self.kbkey = Table('kbkey', meta,
            Column('keycode', Integer, nullable=False),
            Column('cmactioncode', Integer, nullable=True),
            Column('keyvalue', String(10), nullable=True),
            PrimaryKeyConstraint('keycode', name='pk_kbkey')
        )
        self.kit = Table('kit', meta,
            Column('campaignid', Integer, nullable=False),
            Column('kitid', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('name', String(100), nullable=True),
        )
        self.kitdisc = Table('kitdisc', meta,
            Column('disckind', Integer, nullable=True),
            Column('disctype', Integer, nullable=True),
            Column('discval', Numeric(precision=14, scale=3), nullable=True),
            Column('kitdiscid', Integer, nullable=False, autoincrement=True, primary_key=True),
            Column('kitgoodsid', Integer, nullable=False),
        )
        self.kitgoods = Table('kitgoods', meta,
            Column('goodscode', String(100), nullable=False),
            Column('kitgoodsid', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('kitid', Integer, nullable=False),
            Column('quantity', Numeric(precision=14, scale=3), nullable=True),
        )
        self.macroparameters = Table('macroparameters', meta,
            Column('macroparametercode', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('macroparameterorder', Integer, nullable=True),
            Column('cmactioncode', Integer, nullable=True),
            Column('macrocode', Integer, nullable=False),
        )
        self.menu = Table('menu', meta,
            Column('menucode', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('menuname', String(255), nullable=True),
        )
        self.menuitem = Table('menuitem', meta,
            Column('menuitemcode', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('menuitemorder', Integer, nullable=True),
            Column('cmactioncode', Integer, nullable=True),
            Column('menucode', Integer, nullable=False),
        )
        self.mol = Table('mol', meta,
            Column('code', String(30), nullable=False, primary_key=True),
            Column('login', String(60), nullable=False),
            Column('loginmode', Integer, nullable=True),
            Column('name', String(60), nullable=True),
            Column('password', String(60), nullable=True),
            Column('locked', Integer, nullable=True),
            Column('usergroup', Integer, nullable=True),
            Column('drawer', Integer, nullable=True),
            Column('start', String(8), nullable=True),
        )
        self.mop = Table('mop', meta,
            Column('scode', String(30), nullable=True),
            Column('opcode', Integer, nullable=True),
            Column('turnon', Integer, nullable=True),
            Column('password', String(8), nullable=True),
        )
        self.op = Table('op', meta,
            Column('opcode', Integer, nullable=False, primary_key=True),
            Column('name', String(512), nullable=True),
            Column('sign', Integer, nullable=True),
            Column('mode', Integer, nullable=True),
            Column('opgroup', Integer, nullable=True),
            Column('keypos', Integer, nullable=True),
            Column('password', String(64), nullable=True),
            Column('extcode', String(20), nullable=True),
            Column('logstring', String(64), nullable=True),
        )
        self.remain = Table('remain', meta,
            Column('remaincode', Integer, nullable=False, autoincrement=True, primary_key=True),
            Column('quant', Numeric(precision=19, scale=2), nullable=True),
            Column('timevalid', DateTime, nullable=True),
            Column('aspectvaluesetcode', Integer, nullable=True),
            Column('goodscode', String(255), nullable=True),
        )
        self.role = Table('role', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('name', String(255), nullable=True),
        )
        self.rostat = Table('rostat', meta,
            Column('cashcode', String(30), nullable=True),
            Column('dcode', Integer, nullable=True),
            Column('bcode', String(20), nullable=True),
            Column('code', Numeric(precision=20,scale=0), nullable=True),
            Column('quant', Numeric(precision=13, scale=3), nullable=True),
            Column('dt', TIMESTAMP, nullable=True),
            Column('policy', Integer, nullable=True),
            Column('mode1', Integer, nullable=True),
        )
        self.sdabs = Table('sdabs', meta,
            Column('disc_val', Numeric(precision=13, scale=2), nullable=True),
            Column('disc_name', String(32), nullable=True),
        )
        self.sdperc = Table('sdperc', meta,
            Column('disc_val', Numeric(precision=5, scale=2), nullable=True),
            Column('disc_name', String(32), nullable=True),
        )
        self.settings = Table('settings', meta,
            Column('name', String(128), nullable=False, primary_key=True),
            Column('val', String(128), nullable=True),
        )
        self.spindex = Table('spindex', meta,
            Column('ind_val', Numeric(precision=5, scale=2), nullable=True),
            Column('ind_name', String(32), nullable=True),
        )
        self.sprices = Table('sprices', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('name', String(32), nullable=True),
            Column('type', Integer, nullable=True),
            Column('inp_mode', Integer, nullable=True),
            Column('force_mode', Integer, nullable=True),
            Column('ext_mode', Integer, nullable=True),
            Column('cmp_mode', Integer, nullable=True),
            Column('price', Numeric(precision=13, scale=2), nullable=True),
            Column('sindex', Integer, nullable=True),
            Column('disc_perc', Numeric(precision=5, scale=2), nullable=True),
            Column('disc_abs', Numeric(precision=13, scale=2), nullable=True),
            Column('ext_proc', String(80), nullable=True),
        )
        self.tgrlink = Table('tgrlink', meta,
            Column('bcode', String(20), nullable=True),
            Column('class', Integer, nullable=True),
            Column('sclass', Integer, nullable=True),
        )
        self.timeperiod = Table('timeperiod', meta,
            Column('campaignid', Integer, nullable=True),
            Column('date_end', Date, nullable=True),
            Column('date_start', Date, nullable=True),
            Column('day_end', Integer, nullable=True),
            Column('day_start', Integer, nullable=True),
            Column('timeperiodid', Integer, nullable=False, primary_key=True, autoincrement=True),
            Column('time_end', Time(timezone=False), nullable=True),
            Column('time_start', Time(timezone=False), nullable=True),
        )
        self.tmc = Table('tmc', meta,
            Column('bcode', String(20), nullable=False, primary_key=True),
            Column('vatcode1', Integer, nullable=True),
            Column('vatcode2', Integer, nullable=True),
            Column('vatcode3', Integer, nullable=True),
            Column('vatcode4', Integer, nullable=True),
            Column('vatcode5', Integer, nullable=True),
            Column('dcode', Integer, nullable=True),
            Column('name', String(200), nullable=True),
            Column('articul', String(200), nullable=True),
            Column('cquant', Numeric(precision=13, scale=3), nullable=True),
            Column('measure', Integer, nullable=True),
            Column('pricetype', Integer, nullable=True),
            Column('price', Numeric(precision=13, scale=2), nullable=True),
            Column('minprice', Numeric(precision=13, scale=2), nullable=True),
            Column('valcode', Integer, nullable=True),
            Column('quantdefault', Numeric(precision=13, scale=3), nullable=True),
            Column('quantlimit', Numeric(precision=13, scale=3), nullable=True),
            Column('ostat', Integer, nullable=True),
            Column('links', Integer, nullable=True),
            Column('quant_mode', Integer, nullable=True),
            Column('bcode_mode', Integer, nullable=True),
            Column('op_mode', Integer, nullable=True),
            Column('dept_mode', Integer, nullable=True),
            Column('price_mode', Integer, nullable=True),
            Column('tara_flag', Integer, nullable=True),
            Column('tara_mode', Integer, nullable=True),
            Column('tara_default', String(20), nullable=True),
            Column('unit_weight', Numeric(precision=13, scale=3), nullable=True),
            Column('code', String(100), nullable=True),
            Column('aspectschemecode', Integer, nullable=True),
            Column('aspectvaluesetcode', Integer, nullable=True),
            Column('aspectusecase', Integer, nullable=True),
            Column('aspectselectionrule', Integer, nullable=True),
            Column('extendetoptions', String(8000), nullable=True),
            Column('groupcode', String(100), nullable=True),
        )
        self.tmcext = Table('tmcext', meta,
            Column('bcode', String(20), nullable=True),
            Column('code', Numeric(precision=20,scale=0), nullable=True),
            Column('name', String(120), nullable=True),
        )
        self.tmcgroups = Table('tmcgroups', meta,
            Column('groupcode', String(100), nullable=False, primary_key=True),
            Column('parentgroupcode', String(100), nullable=True),
            Column('groupname', String(200), nullable=True),
        )
        self.tmclink = Table('tmclink', meta,
            Column('bcode', String(20), nullable=True),
            Column('bcodeslave', String(20), nullable=True),
            Column('quantslave', Numeric(precision=13, scale=3), nullable=True),
            Column('trans', Integer, nullable=True),
        )
        self.tmcsalerestrict = Table('tmcsalerestrict', meta,
            Column('salerestrictcode', Integer, nullable=False, autoincrement=True, primary_key=True),
            Column('tmccode', String(100), nullable=True),
            Column('datebeg', Date, nullable=True),
            Column('dateend', Date, nullable=True),
            Column('daysbeg', Integer, nullable=True),
            Column('daysend', Integer, nullable=True),
            Column('timebeg', Time(timezone=False), nullable=True),
            Column('timeend', Time(timezone=False), nullable=True),
        )
        self.tostat = Table('tostat', meta,
            Column('dcode', Integer, nullable=True),
            Column('bcode', String(20), nullable=True),
            Column('code', Numeric(precision=20,scale=0), nullable=True),
            Column('quant', Numeric(precision=13, scale=3), nullable=True),
            Column('dt', TIMESTAMP, nullable=True),
            Column('cashcode', String(30), nullable=True),
        )
        self.tprices = Table('tprices', meta,
            Column('tindex', Integer, nullable=False, primary_key=True),
            Column('bcode', String(20), nullable=False, primary_key=True),
            Column('price', Numeric(precision=13, scale=2), nullable=True),
            Column('name', String(40), nullable=True),
        )
        self.tquick = Table('tquick', meta,
            Column('qcode', String(8), nullable=True),
            Column('bcode', String(20), nullable=True),
            Column('bquant', Numeric(precision=7, scale=3), nullable=True),
        )
        self.units = Table('units', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('name', String(8), nullable=True),
            Column('flag', Integer, nullable=True),
        )
        self.valut = Table('valut', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('stat', Integer, nullable=True),
            Column('chr', String(4), nullable=True),
            Column('name', String(32), nullable=True),
            Column('type', String(1), nullable=True),
            Column('rate', Numeric(precision=13, scale=4), nullable=True),
            Column('ratecb', Numeric(precision=13, scale=4), nullable=True),
            Column('decpoint', Integer, nullable=True),
            Column('round', Integer, nullable=True),
            Column('mode', Integer, nullable=True),
            Column('drawer', Integer, nullable=True),
            Column('discascheme', Integer, nullable=True),
            Column('ishidden', Integer, nullable=True),
            Column('operation', Integer, nullable=True),
            Column('payprocmodule', String(4096), nullable=True),
        )
        self.vat = Table('vat', meta,
            Column('code', Integer, nullable=False, primary_key=True),
            Column('name', String(32), nullable=True),
            Column('rate', Numeric(precision=5, scale=2), nullable=True),
            Column('mode', Integer, nullable=True),
        )


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        primary_key_name = table.primary_key.name
        for suffix in [ '1', '2' ]:
            if primary_key_name:
                table.primary_key.name = primary_key_name + suffix
            table.name = basename + suffix
            table.create()

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    # Operations to reverse the above upgrade go here.
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.drop()
