# -*- coding: utf-8 -*-

from sqlalchemy import *
from migrate.changeset.constraint import PrimaryKeyConstraint
from sqlalchemy.engine import reflection
from dictionaries.versions import views_helper

aspectValueSetTableName = 'aspectvalueset'
aspectValueSetValuesTableName = 'aspectvaluesetvalues'
tmcCodeIndexName = ['aspectvaluesetidxtmc_code', 'aspectvaluesetvaluesidxtmc_code']
productSectionCodeColumnName = 'productsectioncode'
productSectionCodeIndexName = ['aspectvaluesetidxproductsectioncode', 'aspectvaluesetvaluesidxproductsectioncode']
aspectValueSetCodeColumnName = 'aspectvaluesetcode'
aspectValueSetAspectSchemeCodeColumnName = 'aspectschemecode'
aspectValueSetValuesPrimaryKey = ('aspectcode', 'aspectvaluecode', 'aspectvaluesetcode', 'productsectioncode', 'tmc_code')
tmcCodeColumnName = 'tmc_code'

def upgrade(migrate_engine):
    # создание индекса для колонки tmc_code
    views_helper.create_index(migrate_engine, aspectValueSetTableName, 'tmc_code', tmcCodeIndexName[0])

    # создание колонки productsectioncode в таблице aspectvalueset и индекса для этой колонки
    views_helper.create_column(migrate_engine, aspectValueSetTableName, productSectionCodeColumnName, String(10))
    views_helper.create_index(migrate_engine, aspectValueSetTableName, productSectionCodeColumnName, productSectionCodeIndexName[0])

    # создание колонки productsectioncode в таблице aspectvaluesetvalues и индекса для этой колонки
    views_helper.create_column(migrate_engine, aspectValueSetValuesTableName, productSectionCodeColumnName, String(10), isNullable=False, default='')
    views_helper.create_index(migrate_engine, aspectValueSetValuesTableName, productSectionCodeColumnName, productSectionCodeIndexName[1])

    # создание колонки tmc_code в таблице aspectvaluesetvalues и индекса для этой колонки
    views_helper.create_column(migrate_engine, aspectValueSetValuesTableName, tmcCodeColumnName, String(255), isNullable=False, default='')
    views_helper.create_index(migrate_engine, aspectValueSetValuesTableName, tmcCodeColumnName, tmcCodeIndexName[1])

    # редактирование колонки aspectschemecode в таблице aspectvalueset так, чтобы она могла принимать значения NULL
    views_helper.alter_column(migrate_engine, aspectValueSetTableName, aspectValueSetAspectSchemeCodeColumnName, isNullable=True)

    insp = reflection.Inspector.from_engine(migrate_engine)
    for table in views_helper.getTables(MetaData(bind = migrate_engine), aspectValueSetValuesTableName):
        if (len(insp.get_pk_constraint(table.name)['constrained_columns'])):
            continue
        PrimaryKeyConstraint(*aspectValueSetValuesPrimaryKey, table=table).create()
    views_helper.update_dic_view(migrate_engine, aspectValueSetValuesTableName)

def downgrade(migrate_engine):
    insp = reflection.Inspector.from_engine(migrate_engine)
    for table in views_helper.getTables(MetaData(bind = migrate_engine), aspectValueSetValuesTableName):
        if (not len(insp.get_pk_constraint(table.name)['constrained_columns'])):
            continue
        PrimaryKeyConstraint(*aspectValueSetValuesPrimaryKey, table=table).drop()
    views_helper.update_dic_view(migrate_engine, aspectValueSetValuesTableName)

    # редактирование колонки aspectschemecode в таблице aspectvalueset так, чтобы она не могла принимать значения NULL
    views_helper.alter_column(migrate_engine, aspectValueSetTableName, aspectValueSetAspectSchemeCodeColumnName, isNullable=False)

    # удаление колонки tmc_code в таблице aspectvaluesetvalues и индекса для этой колонки
    views_helper.drop_index(migrate_engine, aspectValueSetValuesTableName, tmcCodeIndexName[1])
    views_helper.drop_column(migrate_engine, aspectValueSetValuesTableName, tmcCodeColumnName, String(255))

    # удаление индекса колонки productsectioncode в таблице aspectvaluesetvalues и самой колонки
    views_helper.drop_index(migrate_engine, aspectValueSetValuesTableName, productSectionCodeIndexName[1])
    views_helper.drop_column(migrate_engine, aspectValueSetValuesTableName, productSectionCodeColumnName, String(10))

    # удаление индекса колонки productsectioncode в таблице aspectvalueset и  самой колонки
    views_helper.drop_index(migrate_engine, aspectValueSetTableName, productSectionCodeIndexName[0])
    views_helper.drop_column(migrate_engine, aspectValueSetTableName, productSectionCodeColumnName, String(10))

    # удаление индекса для колонки tmc_code
    views_helper.drop_index(migrate_engine, aspectValueSetTableName, tmcCodeIndexName[0])
