from sqlalchemy import *
from migrate import *
import os
import time
import re
try:
    from sqlalchemy.dialects.mssql.pyodbc import MSDialect_pyodbc
except:
    from sqlalchemy.databases.mssql import MSSQLDialect_pyodbc as MSDialect_pyodbc

meta = MetaData()

def AlterTableAddColumn(migrate_engine, table, addColumn):
    t0 = time.time()
    meta.bind = migrate_engine

    if not(isMasterTable(migrate_engine, table)):
        print("Truncate slaveTable...")
        migrate_engine.execute('TRUNCATE %s' %table.name)
        addColumn.create(table)
        return

    alterTableName = table.name
    tableNameWithoutIndex = alterTableName + 'WithoutIndex'
    createTempTable(migrate_engine, tableNameWithoutIndex, alterTableName)
    tableWithoutIndex = Table(tableNameWithoutIndex, meta, autoload=True)

    if not (addColumn.name in tableWithoutIndex.columns):
        addColumn.create(tableWithoutIndex)

    #Create table with all indexes
    tableNameWithIndex = alterTableName + 'WithIndex'
    createTempTable(migrate_engine, tableNameWithIndex, tableNameWithoutIndex)
    dropIndexes(tableWithoutIndex)

    insertQuery = 'INSERT INTO %s SELECT *, null FROM %s' %(tableWithoutIndex, alterTableName)

    copyData(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, insertQuery, t0)

def AlterTableDropColumn(migrate_engine, alterTable, dropColumn, masterTableName):
    t0 = time.time()
    meta.bind = migrate_engine

    if not(masterTableName == alterTable.name):
        print("Truncate slaveTable...")
        migrate_engine.execute('TRUNCATE %s' %alterTable.name)
        dropColumn.drop(alterTable)
        return

    alterTableName = alterTable.name
    tableNameWithoutIndex = alterTableName + 'WithoutIndex'
    createTempTable(migrate_engine, tableNameWithoutIndex, alterTableName)
    tableWithoutIndex = Table(tableNameWithoutIndex, meta, autoload=True)

    for column in tableWithoutIndex.columns:
        if dropColumn.name == column.name:
            column.drop()
            break

    #Create table with all indexes
    tableNameWithIndex = alterTableName + 'WithIndex'
    createTempTable(migrate_engine, tableNameWithIndex, tableNameWithoutIndex)
    dropIndexes(tableWithoutIndex)

    # Prepare Insert statement   
    columnsName = []
    for column in alterTable.columns:
        if column.name in tableWithoutIndex.columns:
            columnsName.append(column.name)
    columnsNameStr = ', '.join(columnsName)

    insertQuery = 'INSERT INTO %s SELECT %s FROM %s' %(tableNameWithoutIndex, columnsNameStr, alterTableName)

    copyData(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, insertQuery, t0)

def AlterTableAddIndex(migrate_engine, table, column_name, index_name):
    t0 = time.time()
    meta.bind = migrate_engine

    if not(isMasterTable(migrate_engine, table)):
        print("Truncate slaveTable...")
        migrate_engine.execute('TRUNCATE %s' %table.name)
        for column in table.columns:
            if column_name == column.name:
                Index(index_name, column, unique=False).create()
                break
        return

    alterTableName = table.name
    tableNameWithoutIndex = alterTableName + 'WithoutIndex'
    createTempTable(migrate_engine, tableNameWithoutIndex, alterTableName)
    tableWithoutIndex = Table(tableNameWithoutIndex, meta, autoload=True)

    if index_name not in [ind.name for ind in tableWithoutIndex.indexes]:
        for column in tableWithoutIndex.columns:
            if column_name == column.name:
                Index(index_name, column, unique=False).create()
                break

    #Create table with all indexes
    tableNameWithIndex = alterTableName + 'WithIndex'
    createTempTable(migrate_engine, tableNameWithIndex, tableNameWithoutIndex)
    dropIndexes(tableWithoutIndex)

    insertQuery = 'INSERT INTO %s SELECT * FROM %s' %(tableWithoutIndex, alterTableName)

    copyData(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, insertQuery, t0)

def AlterTableDropIndex(migrate_engine, alterTable, index_name, masterTableName):
    t0 = time.time()
    meta.bind = migrate_engine

    if not(masterTableName == alterTable.name):
        print("Truncate slaveTable...")
        migrate_engine.execute('TRUNCATE %s' %alterTable.name)
        for index in alterTable.indexes:
            if index_name == index.name:
                index.drop()
                break
        return

    alterTableName = alterTable.name
    tableNameWithoutIndex = alterTableName + 'WithoutIndex'
    createTempTable(migrate_engine, tableNameWithoutIndex, alterTableName)
    tableWithoutIndex = Table(tableNameWithoutIndex, meta, autoload=True)

    for index in tableWithoutIndex.indexes:
        if index_name == index.name:
            index.drop()
            tableWithoutIndex.indexes.remove(index)
            break
    #Create table with indexes
    tableNameWithIndex = alterTableName + 'WithIndex'
    createTempTable(migrate_engine, tableNameWithIndex, tableNameWithoutIndex)
    dropIndexes(tableWithoutIndex)

    # Prepare Insert statement   
    columnsName = []
    for column in alterTable.columns:
        if column.name in tableWithoutIndex.columns:
            columnsName.append(column.name)
    columnsNameStr = ', '.join(columnsName)

    insertQuery = 'INSERT INTO %s SELECT %s FROM %s' %(tableNameWithoutIndex, columnsNameStr, alterTableName)

    copyData(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, insertQuery, t0)

def copyFiles(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, copyInTemp):
    dataDir = migrate_engine.execute('select @@datadir').fetchone()[0]
    #print "dbName"
    url = migrate_engine.url
    leftPart = str(url).partition("?")[0]
    lastPosition = leftPart.rfind("/")
    dbName = leftPart[lastPosition+1:len(leftPart)]
    #print re.search("\/(\w*)\?", url) does not work

    # Rename files
    migrate_engine.execute('FLUSH TABLES')

    tableWithoutIndexPath = dataDir + dbName + '/' + tableNameWithoutIndex
    tableWithIndexPath = dataDir + dbName + '/' + tableNameWithIndex
    alterTablePath = dataDir + dbName + '/' + alterTableName
    try:
        if copyInTemp:
            os.rename(tableWithIndexPath + '.frm', tableWithoutIndexPath + '.frm')
            os.rename(tableWithIndexPath + '.MYI', tableWithoutIndexPath + '.MYI')
            os.remove(tableWithIndexPath + '.MYD')
        else:
            os.rename(tableWithoutIndexPath + '.MYD', alterTablePath + '.MYD')
            os.rename(tableWithoutIndexPath + '.frm', alterTablePath + '.frm')
            os.rename(tableWithoutIndexPath + '.MYI', alterTablePath + '.MYI')
    except OSError as e:
        print('Error rename or delete files: ' + str(e))
        migrate_engine.execute('DROP TABLE IF EXISTS %s' %tableNameWithoutIndex)
        migrate_engine.execute('DROP TABLE IF EXISTS %s' %tableNameWithIndex)
        return False

    return True

def createTempTable(migrate_engine, nameNewTable, nameLikeTable):
    try:
        migrate_engine.execute('SHOW CREATE TABLE %s' %nameNewTable)
        migrate_engine.execute('DROP TABLE IF EXISTS %s' %nameNewTable)
    except:
        pass
    migrate_engine.execute('CREATE TABLE %s LIKE %s' %(nameNewTable, nameLikeTable))

def dropIndexes(table):
    for index in table.indexes:
        index.drop()

    primaryKey = list(table.primary_key)
    if len(primaryKey) != 0:
        PrimaryKeyConstraint(primaryKey[0]).drop()

def copyData(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, insertQuery, startTime):
    print("Copy data...")
    migrate_engine.execute(insertQuery)
    print("Copy data end: " + format(time.time() - startTime, "0.5f"))

    if not (copyFiles(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, "", True)):
        return

    print("Repair table...")
    migrate_engine.execute('REPAIR TABLE %s' %tableNameWithoutIndex)

    if not (copyFiles(migrate_engine, tableNameWithoutIndex, tableNameWithIndex, alterTableName, False)):
        return
    print("Time elapsed: " + format(time.time() - startTime, "0.5f"))

# В python 3.4 проблемы с циклическим импортом, поэтому пришлось продублировать функцию из views_helper
def get_table_name(engine,name):
    conn = engine.connect()
    if type(conn.dialect) is not MSDialect_pyodbc:
        create_view = conn.execute('show create view %s' % name).fetchone()[1]
        m = re.match('.*(%s[12]).*' % name, create_view)
        return m.group(1)

def isMasterTable(engine, table):
    masterTableName = get_table_name(engine, table.name[0:len(table.name)-1])
    return masterTableName == table.name
